
create domain email text
	check ( VALUE ~ $x$^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$$x$ );

comment on domain email is 'This domain is for simple format validation of e-mails.';

create table access_levels (
	level char(1) not null primary key,
	level_name text not null unique check ( length(level_name) between 3 and 20 ),
	level_description text not null
);

comment on table access_levels is 'holds the various access control levels for reference.';

create table privileges (
	privilege text not null unique check ( length(privilege) between 4 and 50 ),
	description text
);

comment on table privileges is 'a list of application privileges which can be assigned to various privilege levels.';

create table access_level_privileges (
	level char(1) not null references access_levels (level)
		on update cascade on delete cascade,
	privilege text not null references privileges (privilege)
		on update cascade on delete cascade,
	constraint access_level_privileges_key unique ( level, privilege ) 
);

comment on table access_level_privileges is 'joins the list of privileges to the list of access levels';

create table users (
	first_name  text not null check ( length(first_name) between 1 and 40 ), 
	last_name   text not null check ( length(last_name) between 2 and 30 ),
	login	    text not null unique check ( length(login) between 4 and 30 ), 
	password    text not null check ( length(login) between 6 and 30 ),
	email       email not null unique,
	description text,
	icon	    text,
	level	    char(1) not null default 'u' references access_levels (level)
			on update cascade on delete set default,
	active	    boolean not null default TRUE,
	profile	    xml
);
	
comment on table users is 'holds user login, descriptive and profile information for all users.';

create table forums (
	forum	SERIAL not null unique,
	name       text not null unique check ( length (name) between 3 and 100 ),
	description text,
	created		timestamp with time zone not null default current_timestamp,
	owner		text not null references users (login) on update cascade
			on delete cascade	
);

comment on table forums is 'a name and description table for the top-level forums or categories';

create table threads (
	thread 	    SERIAL not null unique,
	name	    text not null check ( length (name) between 3 and 100 ),
	description  text,
	forum 	    integer not null references forums(forum)
		    on update cascade on delete cascade,
	created		timestamp with time zone not null default current_timestamp,
	owner		text not null references users (login) on update cascade
			on delete cascade,
	constraint threads_key unique (thread, name)
);

comment on table threads is 'a name and description table for the thread information, that is the master post in a discussion';

create table flags (
	flag	char(1) unique not null,
	description	text not null check ( length (description) between 3 and 100 ),
	icon	text not null
);

comment on table flags is 'reference table for the little graphic flags for each post.  assumes a small quantity of these.';

create table posts (
	post	SERIAL not null unique,
	thread	integer not null references threads(thread)
		on delete cascade on update cascade,
	parent_post	integer references posts(post)
			on delete cascade on update cascade,
	created		timestamp with time zone not null default current_timestamp,
	owner		text not null references users (login) on update cascade
			on delete cascade,
	content		text not null,
	flag		char(1) references flags(flag) 
			on update cascade on delete set null,
	constraint posts_key unique (thread, created, owner)
);

comment on table posts is 'the actual posts with all post content';
create table messages ( 
	message	SERIAL not null unique,
	sender	text not null references users(login) 
		on delete cascade on update cascade,
	sent	timestamp with time zone not null default current_timestamp,
	subject text not null check (length(subject) between 3 and 200 ),
	content text not null,
	constraint messages_key unique ( sender, sent, subject )
);

comment on messages is 'private messages between users';
	
create table message_recipients (
	message int not null references messages(message)
		on delete cascade on update cascade,
	recipient text not null references users(login)
		on delete cascade on update cascade,
	constraint message_recipients_key unique ( message, recipient )
);

comment on message_recipients is 'the users receiving each message';

		
	